// World Map drupal module.
// Nikolay Grachev (Granik).

// Usage: worldMap(optionsObj);
import {select} from "d3-selection";
import {zoomTransform} from "d3-zoom"; // do not remove!

window.worldMap = ( options => {
  // Default options values.
  const defaults = {
    containerSelector: null,
    resetBtnSelector: '.worldmap__back',
    dropdownSelector: '.worldmap__dropdown',
    zoomDuration: 1000,
    resetOnClickOutside: false,
    showTooltips: false,
  };

  // Gap surrounds selected country.
  const viewGap = 40;
  // Container sizes.
  const containerSizeData = {};
  // Options (take defaults into account).
  const optionsActual = Object.assign({}, defaults, options);
  // Main enclosing element.
  const mainContainerElement = document.querySelector(optionsActual.containerSelector);
  // Country select list.
  const dropdownCountrySelect = mainContainerElement.querySelector(optionsActual.dropdownSelector);
  // TRUE if no select found on the map.
  const noSelect = dropdownCountrySelect == null;
  // SVG
  const svg = document.querySelector(optionsActual.containerSelector + ' .worldmap__svg');
  // Initial view zone.
  const viewBoxInitial = svg.getAttribute('viewBox');
  // Initial X,Y coords of view zone.
  const xInit = ''.split.call(viewBoxInitial, ' ')[0];
  const yInit = ''.split.call(viewBoxInitial, ' ')[1];
  // Initial width, height.
  const widthInit = ''.split.call(viewBoxInitial, ' ')[2];
  const heightInit = ''.split.call(viewBoxInitial, ' ')[3];
  // Set zoomable flag value.
  const zoomable = mainContainerElement.getAttribute('data-zoom') === 'on';
  // Is autozoom enabled.
  const autoZoom = mainContainerElement.getAttribute('data-autozoom') === 'on';
  // D3 objects of the current map.
  const svgD3object = select(optionsActual.containerSelector + ' svg');
  const countriesD3object = svgD3object.selectAll('path');
  const countriesPathTags = mainContainerElement.querySelectorAll(`path`);
  // Set click handler for all countries.
  countriesD3object.on('click', _clicked);

  let availableCountries = [];

  [].forEach.call(countriesPathTags, (country) => {
    if (country.classList.contains('is-selected')) {
      // Set available countries.
      availableCountries.push(country);
    }
  });

  if (autoZoom && availableCountries.length === 1) {
    _zoomIn(availableCountries.pop());
  }

  // Set change handler for the dropdown.
  if (!noSelect) {
    dropdownCountrySelect
      .addEventListener('change', _getCountryByDropdown);
  }
  select(optionsActual.containerSelector + ' ' + optionsActual.resetBtnSelector).on('click', _reset);
  // Set container data (width, height).
  _setWidthHeight();
  // On window resize - recalculate container size.
  window.addEventListener('resize', _setWidthHeight);

  // On click somewhere outside - reset the map.
  if (optionsActual.resetOnClickOutside) {
    mainContainerElement.addEventListener('click', _reset);
  }

  // Compute width, height of the containing element.
  function _setWidthHeight() {
    const style = getComputedStyle(mainContainerElement);
    const widthPx = style.getPropertyValue('width');
    const heightPx = style.getPropertyValue('height');
    containerSizeData.width = parseInt(widthPx, 10);
    containerSizeData.height = parseInt(heightPx, 10)
    if (containerSizeData.width < 850) {
      containerSizeData.width = 1000;
      containerSizeData.height = 500;
    }
    svg.setAttribute('width', parseInt(widthPx, 10));
    svg.setAttribute('height', parseInt(heightPx, 10));
  }

  // Click handler for countries.
  function _clicked(event) {
    event.stopPropagation();
    const country = event.currentTarget;

    if (optionsActual.resetOnClickOutside && !country.classList.contains('is-current')) {
      _reset();
    }

    if (!country.classList.contains('is-selected')) {
      // If country is not available.
      return false;
    }

    if (!optionsActual.resetOnClickOutside) {
      [].forEach.call(countriesPathTags, (pathTag) => {
        pathTag.classList.remove('is-current');
      });
    }

    country.classList.add('is-current');
    if (!noSelect) {
      const option = dropdownCountrySelect.querySelector(`option[value=${country.getAttribute('data-id')}]`);
        option.selected = true;
    }

    if (zoomable) {

      _zoomIn(country);

      // pathD3object.on('click', _linked);
      setTimeout(() => {
        _linked(null, country);
      }, optionsActual.zoomDuration);

      return;
    }

    _linked(null, country);

  }

  // Reset map state.
  function _reset(event) {

    if (event) {
      event.stopPropagation();
    }

    if (!noSelect) {
      dropdownCountrySelect.selectedIndex = 0;
    }

    if (zoomable && !mainContainerElement.classList.contains('is-zoomed')) {
      return;
    }

    mainContainerElement.classList.remove('is-zoomed');

    [].forEach.call(countriesPathTags, (pathTag) => {
      pathTag.classList.remove('is-current');
    });

    svgD3object.transition()
      .duration(optionsActual.zoomDuration)
      .attr('viewBox', `${xInit} ${yInit} ${widthInit} ${heightInit}`);

    countriesD3object.on('click', null);
    countriesD3object.on('click', _clicked);
  }

  function _zoomIn(country) {
    let d = country.getAttribute('d');
    let countryD3object = select(optionsActual.containerSelector + ` path[d='${d}']`);
    mainContainerElement.classList.add('is-zoomed');
    // Add .current class to the country.
    let bBox = countryD3object.node().getBBox();
    let x0 = bBox.x - viewGap;
    let y0 = bBox.y - viewGap;
    let width = bBox.width + 2*viewGap;
    let height = bBox.height + 2*viewGap;

    // Zooming.
    svgD3object.transition()
      .duration(optionsActual.zoomDuration)
      .attr('viewBox', `${x0} ${y0} ${width} ${height}`);
  }

  // Make country linked.
  function _linked(event, elem = null) {
    let url;

    if (elem instanceof HTMLElement || elem instanceof EventTarget) {
      // If elem is provided.
      url = elem.getAttribute('data-url');
    }

    if (event) {
      // If this function is a callback.
      url = event.currentTarget.getAttribute('data-url');
    }
    if (!url) {
      return false;
    }

    location.href = url;

  }

  // Select country with dropdown list.
  function _getCountryByDropdown(event) {
    const countryCode = event.currentTarget.value;

    //Get <path> element of the current country.
    const path = mainContainerElement.querySelector(`path[data-id='${countryCode}']`);

    // Trigger click event.
    setTimeout(() => {
      path.dispatchEvent(new MouseEvent('click'));
    });
  }

  if (options.showTooltips) {
    svg.addEventListener('mousemove', _showTooltip);
  }

  // Tooltip (event handler for mousemove).
  function _showTooltip(e) {
    const name = e.target.getAttribute('data-name');
    const tooltip = mainContainerElement.querySelector('.worldmap__tooltip');
    if (!name) {
      // If not on the country.
      tooltip.style.display = 'none';
      return;
    }
    // If on country hover.
    tooltip.style.display = '';

    const x = e.offsetX + 10;
    const y = e.offsetY + 10;
    tooltip.innerText = name;
    tooltip.style.left = x + "px";
    tooltip.style.top = y + "px";
  }

});



