<?php

namespace Drupal\worldmap\Plugin\Block;

use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\Yaml\Yaml;

/**
 * Provides a 'World Map' Block.
 *
 * @Block(
 *   id = "worldmap",
 *   admin_label = @Translation("World Map"),
 *   category = @Translation("World Map"),
 * )
 */
class WorldMapBlock extends BlockBase {

  /**
   * Path to country data.
   */
  const COUNTRY_YAML = __DIR__ . '/../../map_data.yml';


  /**
   * {@inheritDoc}
   */
  public function defaultConfiguration() {
    return [
      'countries' => [],
      'links' => [],
      'zoom' => TRUE,
      'display_dropdown' => TRUE,
      'display_zoom_out_icon' => TRUE,
      'auto_zoom' => FALSE,
      'reset_on_click_outside' => FALSE,
      'tooltips' => FALSE,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $form['country_settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Settings'),
    ];

    $form['country_settings']['zoom'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Zoom-in to country on-click'),
      '#default_value' => !empty($this->configuration['zoom']) ? $this->configuration['zoom'] : NULL,
    ];

    $form['country_settings']['display_dropdown'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display country dropdown list'),
      '#default_value' => !empty($this->configuration['display_dropdown']) ? $this->configuration['display_dropdown'] : NULL,
    ];

    $form['country_settings']['advanced'] = [
      '#type' => 'details',
      '#title' => t('Advanced Settings'),
    ];

    $form['country_settings']['advanced']['auto_zoom'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Auto zoom'),
      '#description' => t('Applies only if one country is available'),
      '#default_value' => !empty($this->configuration['auto_zoom']) ? $this->configuration['auto_zoom'] : NULL,
    ];

    $form['country_settings']['advanced']['reset_on_click_outside'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Reset zoom on click outside'),
      '#default_value' => !empty($this->configuration['reset_on_click_outside']) ? $this->configuration['reset_on_click_outside'] : NULL,
    ];

    $form['country_settings']['advanced']['tooltips'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show country names as tooltips'),
      '#default_value' => !empty($this->configuration['tooltips']) ? $this->configuration['tooltips'] : NULL,
    ];

    $form['country_settings']['advanced']['display_zoom_out_icon'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Display "zoom-out" icon'),
      '#default_value' => !empty($this->configuration['display_zoom_out_icon']) ? $this->configuration['display_zoom_out_icon'] : NULL,
    ];

    $country_options = $this->getCountryOptions();
    $form['list'] = [
      '#type' => 'fieldset',
      '#title' => t('Available countries'),
      '#attributes' => [
        'class' => ['worldmap__countrylist'],
      ],
    ];

    foreach ($country_options as $code => $country) {
      $form['list'][$code]['country'] = [
        '#type' => 'checkbox',
        '#title' => $country,
        '#attributes' => [
          'class' => ['worldmap__checkbox'],
        ],
        '#default_value' => !empty($this->configuration['countries'][$code]) ? $this->configuration['countries'][$code] : NULL,
      ];

      $form['list'][$code]['link'] = [
        '#attributes' => [
          'class' => ['worldmap__linkfield'],
        ],
        '#type' => 'textfield',
        '#size' => 42,
        '#maxlength' => 128,
        '#placeholder' => 'https://www.example.com/ OR /path/to/internal-page',
        '#default_value' => !empty($this->configuration['links'][$code]) ? $this->configuration['links'][$code] : NULL,
      ];
    }

    $form['#attached']['library'][] = 'worldmap/backend';

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    foreach ($form_state->getValue('list') as $country_code => $value) {
      $this->configuration['countries'][$country_code] = $value['country'];
      $this->configuration['links'][$country_code] = $value['link'];
    }
    $this->configuration['zoom'] = $form_state->getValue(['country_settings', 'zoom']);
    $this->configuration['display_dropdown'] = $form_state->getValue(['country_settings', 'display_dropdown']);
    $this->configuration['auto_zoom'] = $form_state->getValue(['country_settings', 'advanced', 'auto_zoom']);
    $this->configuration['display_zoom_out_icon'] = $form_state->getValue(['country_settings', 'advanced', 'display_zoom_out_icon']);
    $this->configuration['reset_on_click_outside'] = $form_state->getValue(['country_settings', 'advanced', 'reset_on_click_outside']);
    $this->configuration['tooltips'] = $form_state->getValue(['country_settings', 'advanced', 'tooltips']);
  }


  /**
   * {@inheritdoc}
   */
  public function build() {
    $config = $this->getConfiguration();
    $map_data = $this->getMapData();
    $map_id = uniqid('worldmap');

    return [
      '#theme' => 'worldmap',
      '#map_id' => $map_id,
      '#config' => $config,
      '#map_data' => $map_data,
      '#module_url' => file_create_url(
        drupal_get_path('module', 'worldmap')
      ),
      '#plugin_id' => $this->getPluginId(),
      '#attached' => [
        'library' => [
          'worldmap/frontend',
        ],
      ],
    ];
  }

  /**
   * Parse yaml file.
   *
   * @return array
   */
  protected function parseYamlData() {
    return Yaml::parseFile(static::COUNTRY_YAML);
  }

  /**
   * Get country options.
   *
   * @return array
   */
  protected function getCountryOptions() {
    $options = [];
    $country_list = [];
    \Drupal::moduleHandler()->invokeAll('worldmap_countries', [&$country_list]);
    foreach ($country_list as $code => $country) {
      $options[$code] = $country;
    }
    uasort($options, function ($a, $b) {
      return strcmp($a, $b);
    });
    return $options;
  }

  /**
   * Get map data.
   *
   * @return array
   */
  protected function getMapData() {
    $data = $this->parseYamlData();
    $country_list = [];
    \Drupal::moduleHandler()->invokeAll('worldmap_countries', [&$country_list]);
    foreach ($data as $code => &$country) {
      $country_name_tmp = $country['name'];
      $country['name'] = !empty($country_list[$code]) ? $country_list[$code] : $country_name_tmp;
      $country['selected'] = FALSE;
      if (!empty($this->getConfiguration()['countries'][$code])) {
        $country['selected'] = TRUE;
      }
      $links = $this->getConfiguration()['links'];
      if (!empty($links[$code])) {
        $country['link'] = $links[$code];
      }
    }

    uasort($data, function ($a, $b) {
      return strcmp($a['name'], $b['name']);
    });

    return $data;
  }

}
